(* By Scott D. Stephenson, CIS #70564,677 *)
(* 24 Oct 93 *)

(* Program shell courtesy of Boilerplate (copyright L. David Baldwin) *)
(* Mr Baldwin has written several Pascal tools found on the BPascal SIG *)

(* Animator is a rather crude "toy" program I wrote in a moment of boredom. *)
(* Basically, I wanted to see if I could animate the desktop background screen. *)
(* This version simply draws various things on the desktop window at one second intervals. *)

(* There sure is a lot of room for improvement in this code. *)
(* If you create better animations, and I sure hope you will, *)
(* please post them so we can all enjoy the fun! *)

(* The Animator source code is free to all. *)


program animator;

uses
  Objects, ctl3d, OWindows, ODialogs, WinTypes, WinProcs, Strings, OStdDlgs, OStdWnds, Windos;

{$R animator.RES}
{$I animator.INC}

type
  TMyApp = object(TApplication)
    procedure InitInstance; virtual;
    procedure InitMainWindow; virtual;
  end;

  PMyWindow = ^TMyWindow;
  TMyWindow = object(TDlgWindow)
    procedure   WMTimer (var Msg: TMessage); virtual wm_First + Wm_Timer;
    constructor Init(ATitle : PChar);
    procedure   SetupWindow ; virtual;
    function    GetClassName : PChar; virtual;
    procedure   GetWindowClass(var AWndClass : TWndClass); virtual;
    destructor  Done; virtual;
    { menu selections }
    procedure   menurectangles(var Msg: TMessage); virtual cm_First + mirectangles;
    procedure   menuellipses(var Msg: TMessage); virtual cm_First + miellipses;
    procedure   menulines(var Msg: TMessage); virtual cm_First + mismalllines;
    procedure   menudots(var Msg: TMessage); virtual cm_First + midots;
    procedure   menucolor(var Msg: TMessage); virtual cm_First + micolor;
    procedure   menuabout(var Msg: TMessage); virtual cm_First + miAbout;
    procedure   uncheckall;
  end;

type animationtype = (Dots,Rectangles, Ellipses, SmallLines);
var DesktopRectangle: trect;
var TypeOfAnimation: animationtype;
var colorvalue: byte;

procedure tmywindow.uncheckall;
begin
CheckMenuItem(GetMenu(HWindow),mismalllines,mf_unchecked);
CheckMenuItem(GetMenu(HWindow),miellipses,mf_unchecked);
CheckMenuItem(GetMenu(HWindow),mirectangles,mf_unchecked);
CheckMenuItem(GetMenu(HWindow),midots,mf_unchecked);
InvalidateRect(getdesktopwindow,@DesktopRectangle,true);
end;

procedure   tmywindow.menucolor(var Msg: TMessage);
begin
if colorvalue = 255 then
  begin
  colorvalue:= 0;
  CheckMenuItem(GetMenu(HWindow),micolor,mf_unchecked);
  end
else
  begin
  colorvalue:= 255;
  CheckMenuItem(GetMenu(HWindow),micolor,mf_checked);
  end;
end;

procedure   tmywindow.menuabout(var Msg: TMessage);
begin
messagebox(hwindow,'Animator  (version: 24 Oct 93)' + #10 + #13 +
                   'by Scott D. Stephenson' + #10 + #13 +
                   'CIS ID#      70564,677' + #10 + #13 +
                   'Written in Borland Pascal(tm)'+ #10 + #13 +
                   + #10 + #13 +
                   'Animator is totally, absolutely, unreservedly free.',
                   'About Animator',mb_ok);
end;

procedure   tmywindow.menulines(var Msg: TMessage);
begin
uncheckall;
CheckMenuItem(GetMenu(HWindow),mismalllines,mf_checked);
typeofanimation:= smalllines;
end;

procedure   tmywindow.menurectangles(var Msg: TMessage);
begin
uncheckall;
CheckMenuItem(GetMenu(HWindow),mirectangles,mf_checked);
typeofanimation:= rectangles;
end;

procedure   tmywindow.menudots(var Msg: TMessage);
begin
uncheckall;
CheckMenuItem(GetMenu(HWindow),midots,mf_checked);
typeofanimation:= dots;
end;

procedure   tmywindow.menuellipses(var Msg: TMessage);
begin
uncheckall;
CheckMenuItem(GetMenu(HWindow),miellipses,mf_checked);
typeofanimation:= ellipses;
end;

procedure drawrectangles(var dc: hdc);
var r: trect;
begin
copyrect(r,desktoprectangle);
with r do
  begin
  left:= random(right);
  top:= random(bottom);
  right:=  left +random(15);
  bottom:=  top + random(15);
  end;
with r do rectangle(dc,left,top,right,bottom);
end;

procedure drawEllipses(var dc: hdc);
var r: trect;
begin
copyrect(r,desktoprectangle);
with r do
  begin
  left:= random(right);
  top:= random(bottom);
  right:=  left +random(15);
  bottom:=  top + random(15);
  end;
with r do ellipse(dc,left,top,right,bottom);
end;

procedure drawdots(var dc: hdc);
var x: integer;
begin
for x:= 1 to 100 do
setpixel(dc,random(desktoprectangle.right),
            random(desktoprectangle.bottom),
rgb(random(colorvalue),random(colorvalue),random(colorvalue)));
end;

procedure drawSmallLines(var dc: hdc);
begin
with desktoprectangle do moveto(dc,random(right),random(bottom));
with desktoprectangle do lineto(dc,random(right),random(bottom));
end;

procedure animatedesktop;
var desk: hwnd;
    dc: hdc;
    x: integer;
    r: trect;
    oldbrush, brush: hbrush;
    oldpen, pen: hpen;
begin
pen:= createpen(ps_solid,1,rgb(random(colorvalue),random(colorvalue),random(colorvalue)));
brush:= createsolidbrush(rgb(random(colorvalue),random(colorvalue),random(colorvalue)));
desk:= getdesktopwindow;
bringwindowtotop(desk);
dc:= getdc(desk);
oldpen:= SelectObject(dc, pen);
oldbrush:= SelectObject(dc, brush);
case typeofanimation of
  rectangles: drawrectangles(dc);
  ellipses:   drawellipses(dc);
  smalllines: drawsmalllines(dc);
  dots:       drawdots(dc);
  else ;
end;
SelectObject(dc, oldpen);
deleteobject(pen);
SelectObject(dc, oldbrush);
deleteobject(brush);
releasedc(desk,dc);
end;

procedure   TMyWindow.WMTimer (var Msg: TMessage);
begin
animatedesktop;
end;

{-------TMyWindow.Init}
constructor TMyWindow.Init(ATitle : PChar);
begin
TDlgWindow.Init(Nil, 'ANIMATOR');
typeofanimation:= dots;
colorvalue:= 255;
end;

{-------TMyWindow.SetupWindow;}
procedure TMyWindow.SetupWindow;
begin
TDlgWindow.SetupWindow;
setTimer(hwindow,1,1000,nil);
getclientrect(getdesktopwindow,DesktopRectangle);
end;

{-------TMyWindow.GetClassName}
function TMyWindow.GetClassName : PChar;
begin
GetClassName := 'ANIMATOR';
end;

{-------TMyWindow.GetWindowClass}
procedure TMyWindow.GetWindowClass(var AWndClass : TWndClass);
begin
TDlgWindow.GetWindowClass(AWndClass);
with AWndClass do
  begin
  hCursor := LoadCursor(0, PChar(idc_Arrow));
  hIcon := LoadIcon(0, PChar(idi_Application));
  Style := cs_Vredraw or cs_Hredraw;
  end;
end;

{------TMyWindow.Done}
destructor TMyWindow.Done;
var r: trect;
begin
TDlgWindow.Done;
KillTimer(hWindow,1 );
(* hey, we don't want to leave our desktop cluttered after we quit! *)
InvalidateRect(getdesktopwindow,@DesktopRectangle,true);
end;

{------TMyApp.InitMainWindow}
procedure TMyApp.InitMainWindow;
begin
  MainWindow := New(PMyWindow, Init(''));
end;

{------TMyApp.InitInstance}
procedure TMyApp.InitInstance;
begin
TApplication.InitInstance;
hAccTable := 0;
end;

var
  App : TMyApp;
begin
  randomize;
  Ctl3dRegister(HInstance);
  Ctl3dAutoSubclass(HInstance);
  App.Init('MyWindow');
  App.Run;
  App.Done;
  ctl3dUnregister(HInstance);
end.
